//////////
// Popravki v originalni izvorni kodi:
// 1.	Spremenjen je makro eq(), ki klie funkcijo strcmp(). Noben izmed
//		parametrov, ki ju funkcija strcmp() sprejme, ne sme biti NULL, sicer se
//		program zrui. Makro je spremenjen tako, da so NULL argumenti dovoljeni.
// 2.	V datoteki misc/random.c je na novo napisan generator nakljunih tevil,
//		zato ne potrebujemo ve funkcij srand() in rand(). Temu ustrezno je
//		spremenjena koda. Dodani sta extern deklaraciji funkcij ran() in gauss().
// Avtor: Janez Puhan
//////////
// 3.	#ifdef HAS_BCOPY popravi udno obnaanje ob prevajanju. Zadevo je opazil
//		linux-a, ki je prvi portal spice pod linux.
// 4.	#ifndef HAS_SYSERRLIST preprei, da bi se ponovno deklarirala sys_errlist,
//		ki je deklarirana e v stdio.h.
// Avtor: Arpad Burmen
//////////

/**********
Copyright 1990 Regents of the University of California.  All rights reserved.
**********/

#include "port.h"

#ifndef MISC_H
#define MISC_H

#define BSIZE_SP      4096

#ifdef HAS_EXIT1
#  define EXIT_NORMAL 1
#  define EXIT_BAD    0
#else
#  define EXIT_NORMAL 0
#  define EXIT_BAD    1
#endif

#ifdef HAS_CTYPE
#  ifndef isalpha
#    include <ctype.h>
#  endif
#endif

// #define eq(a,b) (!strcmp((a), (b)))	// Originalna koda.
// Makro eq() lahko sedaj sprejema tudi NULL argumente.
#define eq(a,b) (((a) && (b)) ? !strcmp((a), (b)) : ((!(a) && !(b)) ? 1 : 0))
#define eqc(a,b)  (cieq((a), (b)))
#define isalphanum(c)   (isalpha(c) || isdigit(c))
#define hexnum(c) ((((c) >= '0') && ((c) <= '9')) ? ((c) - '0') : ((((c) >= \
        'a') && ((c) <= 'f')) ? ((c) - 'a' + 10) : ((((c) >= 'A') && \
        ((c) <= 'F')) ? ((c) - 'A' + 10) : 0)))

#include "strext.h"

#include "unimem.h"

extern char *copy_internal();
extern char *gettok();
extern void appendc();
extern int scannum();
extern int prefix();
extern int ciprefix();
extern int cieq();
extern void strtolower();
extern int substring();
extern char *tilde_expand( );
extern void cp_printword();

extern char *datestring();
extern double seconds();

extern char *smktemp();

extern void setran(long int);
extern double ran();	// Deklaraciji funkcij za generiranje nakljunih tevil.
extern double gauss();

int isAbsolutePath(char *);
int getCurrentDir(char *, int);
void slashToLocalPath(char *);
void stripFileName(char *);
char *dirCat(char *, char *);

/* Externs from libc */

#ifdef HAS_STDLIB

#  ifndef _STDLIB_INCLUDED
#    define _STDLIB_INCLUDED
#    include <stdlib.h>
#  endif
//////////
// Originalna koda (ne potrebujemo ve, ker imamo nov generator nakljunih tevil):
//	#  ifndef HAS_BSDRAND
//		#    define random	rand
//		#    define srandom	srand
//	#  endif
//////////
#  ifdef HAS_DOSDIRS
char *getcwd( );
#  endif

#else

//////////
// Originalna koda (ne potrebujemo ve, ker imamo nov generator nakljunih tevil):
//	#  ifdef HAS_BSDRAND
//	extern long random();
//	extern void srandom();
//	#  else
//	#    define random	rand
//	#    define srandom	srand
//	#  endif
//////////

// extern char *calloc();
// extern char *malloc();
// extern char *realloc();
extern char *getenv();
extern int errno;
// Vasih sys_errlist ni treba deklarirati, ker je deklarirana e v stdio.h.
#ifndef HAS_SYS_ERRLIST
extern char *sys_errlist[];
#endif
extern char *getenv();
extern char *getwd();
// extern int rand();
// extern int srand();
extern int atoi();
extern int kill();
// extern int getpid();
// extern int qsort();
#  ifdef notdef
extern void exit();
#  endif

#  ifdef HAS_GETCWD
extern char *getcwd( );
#  endif

#  ifdef HAS_CLEARERR
#    ifndef clearerr
extern void clearerr();
#    endif /* clearerr */
#  endif /* HAS_CLEARERR */

#ifndef HAS_BCOPY	// Dodano za prevajanje pod linuxom.
#  ifndef bzero
extern int bzero();
#  endif
#  ifndef bcopy
extern void bcopy();
#  endif
#endif	// HAS_BCOPY

#  ifndef index
#    ifdef HAS_INDEX
extern char *rindex();
extern char *index();
#    else
#      ifdef HAS_STRCHR
extern char *strchr();
extern char *strrchr();
#      else
#      endif
#    endif
#  endif

#endif	/* else STDLIB */

#ifndef HAS_INDEX
#  ifndef index
#    ifdef HAS_STRCHR
#      define	index	strchr
#      define	rindex	strrchr
#    endif
#  endif
#endif

#ifdef HAS_TIME_
#  ifdef HAS_BSDTIME
extern char *timezone();
#  endif
extern char *asctime();
extern struct tm *localtime();
#endif
//////////
// Originalna koda:
//	#ifndef HAS_MEMAVL
//	#  ifdef HAS_RLIMIT_
//		extern char *sbrk();
//	#  endif
//	#endif
//////////
#define false 0
#define true 1

#ifdef HAS_DOSDIRS
typedef	int	DIR;
struct direct {
	int	d_reclen;
	short	d_ino;
	short	d_namelen;
	char	d_name[20];
	};

#  ifdef __STDC__
extern DIR *opendir(char *);
extern struct direct *readdir(DIR *);
#  else
extern DIR *opendir( );
extern struct direct *readdir( );
#  endif

#endif

#ifndef HAS_STRICMP
#define stricmp(a,b) strcasecmp(a,b)
#endif

#endif /* MISC_H */
