// Splona glava za datoteke, ki implementirajo optimize ukaz.
// Avtor: Janez Puhan

#include <stdio.h>
#include "spice.h"
#include "util.h"
#include "ftedefs.h"

struct metoda	// Struktura vsebuje ime trenutno izbrane metode in vrednosti parametrov te metode.
{
	char *ime;	// Ime optimizacijske metode.
	char **imena_numericnih;	// Imena numerinih parametrov metode.
	double *vrednosti_numericnih;	// Vrednosti numerinih parametrov metode.
	int stevilo_numericnih;	// tevilo numerinih parametrov metode.
	char **imena_znakovnih;	// Imena znakovnih parametrov metode.
	char **vrednosti_znakovnih;	// Vrednosti znakovnih parametrov metode.
	int stevilo_znakovnih;	// tevilo znakovnih parametrov metode.
};

struct parameter	// Struktura opisuje eksplicitne omejitve in zaetno vrednost enega parametra. Poleg tega so v skrukturi vsebovani tudi podatki o parametru v programski strukturi SPICE-a.
{
	char *ime;	// Ime optimizacijskega parametra.
	int ime_podano;
	double spodnja_meja;	// Eksplicitne omejitve.
	int spodnja_meja_podana;
	double zgornja_meja;
	int zgornja_meja_podana;
	double zacetna_vrednost;	// Zaetna vrednost parametra, oziroma njegova srednja vrednost pri metodi nakljunega iskanja in normalno razporeditvijo.
	int zacetna_vrednost_podana;
	double srednja_vrednost;	// Srednja vrednost pri normalni razporeditvi.
	int srednja_vrednost_podana;
	double deviacija;	// Standardna deviacija.
	int deviacija_podana;
	int logsweep;	// Parameter v logaritmini skali.
	double originalna_vrednost;	// Vrednost parametra pred prietkom optimizacijske zanke.
};

extern void setparams(); // Postavi parametre na zadnje vrednosti v tabeli rezultati
extern void postavi_parametre(struct wordlist *ukaz);	// Funkcija postavlja parametre optimizacijske zanke.
extern void axis_postavi(struct wordlist *beseda);	// Funkcija postavi parametre metode iskanja po koordinatnih smereh.
extern void complex_postavi(struct wordlist *beseda);	// Funkcija postavi parametre metode omejenih simpleksov.
extern void dfp_postavi(struct wordlist *beseda);	// Funkcija postavi parametre Davidon-Fletcher-Powellove metode.
extern void genetic_postavi(struct wordlist *beseda);	// Funkcija postavi parametre genetskega algoritma.
extern void grid_postavi(struct wordlist *beseda);	// Funkcija postavi parametre metode iskanja po mrei.
extern void hj_postavi(struct wordlist *beseda);	// Funkcija postavi parametre metode iskanja po koordinatnih smereh.
extern void monte_postavi(struct wordlist *beseda);	// Funkcija postavi parametre metode nakljunega iskanja.
extern void newton_postavi(struct wordlist *beseda);	// Funkcija postavi parametre metode iskanja po koordinatnih smereh.
extern void space_postavi(struct wordlist *beseda);	// Funkcija postavi parametre iskanja po celotnem parametrskem prostoru.
extern void powell_postavi(struct wordlist *beseda);	// Funkcija postavi parametre Powellove metode.
extern void steepest_postavi(struct wordlist *beseda);	// Funkcija postavi parametre metode najstrmejega sestopa.
extern void povecaj_polje_rezultatov(int n);					// Preveri ce je dovolj prostora za n rezultatov. Ce ni, poveca polje rezultati.

extern void sprosti_spomin_populacije(int sporocilo);	// Funkcija sprosti spomin, ki ga zavzema trenutna oziroma zadnja populacija v genetskem algoritmu.
extern void sprosti_spomin_izracunanih_tock();	// Funkcija sprosti spomin, ki ga zavzema polje tok izraunanih v prejnjih optimizacijskih postopkih.
extern void sprosti_spomin_metode();	// Funkcija sprosti spomin, ki ga zavzemajo imena in vrednosti parametrov optimizacijske metode. Poleg tega sprosti e populacijo genetskega algoritma, e je le ta ostala.

extern void optimiziraj();	// Funkcija zaene optimizacijsko zanko.
extern void naredi_analize(double *tocka);	// Funkcija se postavi v dano toko parametrskega prostora in izvri vse zahtevane analize.
extern void novi_parametri(double *tocka);	// Funkcija spremeni parametre vezja.
extern int originalna_vrednost(int i);	// Funkcija odita vrednost i-tega optimizacijskega parametra. e parameter ne obstaja vrne ni, drugae ena.
extern int implicitne_omejitve_ok();	// Funkcija preveri izpolnjenost implicitnih omejitev.
extern void unici_vse();	// Funkcija unii vse skupine vektorjev, ki so bile narejene v com_optimize() funkciji.
extern double izracunaj_kriterijsko_funkcijo();	// Funkcija izrauna vrednost kriterijske funkcije.

extern double* izracunaj_pravi_gradient(double *gradient, double *tocka, double vrednost_kriterijske_funkcije);	// Funkcija izrauna gradient kriterijske funkcije v podani toki. Vrne vrednosti odvodov transformacije iz omejenega v neomejen prostor.
extern double popravi_smer(double *smer, double **matrika, double *gradient, double *odvod);	// Funkcija pomnoi gradient kriterijske funkcije s podano matriko in tako popravi smer iskanja minimuma. Vrne dolino zaetnega koraka v izraunani smeri.
extern double normiraj(double *smer, double* odvod);	// Funkcija normira dolino podane smeri in vrne velikost zaetnega koraka v tej smeri.
extern void transformiraj(double *neomejena_tocka, double *omejena_tocka, char *transformacija);	// Funkcija transformira toko iz neomejenega v eksplicitno omejen prostor.

extern double fibonaccijeva_metoda(double *prva_tocka, double konstanta, double prva_vrednost, double tretja_vrednost, double *najboljsa_tocka, double *gradient, double zacetni_korak);	// Funkcija doloi minimum kriterijske funkcije v izbrani smeri po Fibonaccijevi metodi razdelitve intervala. Vrne vrednost kriterijske funkcije v izraunanem minimumu.
extern double fibonaccijeva_metoda_s_transformacijo(double *prva_tocka, double konstanta, double prva_vrednost, double tretja_vrednost, double *najboljsa_tocka, double *gradient, double zacetni_korak);	// Funkcija doloi minimum kriterijske funkcije v izbrani smeri po Fibonaccijevi metodi razdelitve intervala. Pri tem pred izvrevanjem analiz vrednosti parametrov transformira iz noemejenega v eksplicitno omejen prostor. Vrne vrednost kriterijske funkcije v izraunanem minimumu.
extern double metoda_zlatega_reza(double *prva_tocka, double konstanta, double prva_vrednost, double tretja_vrednost, double *najboljsa_tocka, double *gradient, double zacetni_korak);	// Funkcija doloi minimum kriterijske funkcije v izbrani smeri po metodi zlatega reza. Vrne vrednost kriterijske funkcije v izraunanem minimumu.
extern double metoda_zlatega_reza_s_transformacijo(double *prva_tocka, double konstanta, double prva_vrednost, double tretja_vrednost, double *najboljsa_tocka, double *gradient, double zacetni_korak);	// Funkcija doloi minimum kriterijske funkcije v izbrani smeri po metodi zlatega reza. Pri tem pred izvrevanjem analiz vrednosti parametrov transformira iz noemejenega v eksplicitno omejen prostor. Vrne vrednost kriterijske funkcije v izraunanem minimumu.
extern double metoda_kvadratnega_priblizevanja(double *prva_tocka, double konstanta, double prva_vrednost, double druga_vrednost, double tretja_vrednost, double *najboljsa_tocka, double *gradient, double zacetni_korak);	// Funkcija doloi minimum kriterijske funkcije v izbrani smeri po metodi kvadratnega priblievanja. Vrne vrednost kriterijske funkcije v izraunanem minimumu.
extern double metoda_kvadratnega_priblizevanja_s_transformacijo(double *prva_tocka, double konstanta, double prva_vrednost, double druga_vrednost, double tretja_vrednost, double *najboljsa_tocka, double *gradient, double zacetni_korak);	// Funkcija doloi minimum kriterijske funkcije v izbrani smeri po metodi kvadratnega priblievanja. Pri tem pred izvrevanjem analiz vrednosti parametrov transformira iz noemejenega v eksplicitno omejen prostor. Vrne vrednost kriterijske funkcije v izraunanem minimumu.

extern void izpis_rezultatov(double *optimum, double zacetni_cas, double najnizja_vrednost_kriterijske_funkcije);	// Funkcija izpie statistine podatke o optimizacijski zanki in rezultate optimizacije.
extern char *predelaj_ime(char *ime);	// Funkcija predela vhodni niz tako, da izpusti vse znake '@' in ']', znake '[' pa spremeni v '_'. Vhodni niz ostane nespremenjen, za izhodni niz pa je rezerviran nov spomin.

extern void koordinatne_smeri(int *optimize_plot, double zacetni_cas);	// Algoritem metode iskanja po koordinatnih oseh.
extern void omejeni_simpleksi(int *optimize_plot, double zacetni_cas);	// Algoritem metode omejenih simpleksov.
extern void dfp(int *optimize_plot, double zacetni_cas);	// Algoritem Davidon-Fletcher-Powellove metode.
extern void genetski_algoritem(double zacetni_cas);	// Preprost genetski algoritem.
extern void iskanje_po_mrezi(int *optimize_plot, double zacetni_cas);	// Algoritem metode iskanja po mrei.
extern void hj(int *optimize_plot, double zacetni_cas);	// Algoritem Hooke-Jeevesove metode.
extern void nakljucno_iskanje(int *optimize_plot, double zacetni_cas);	// Algoritem metode nakljunega iskanja.
extern void newton(int *optimize_plot, double zacetni_cas);	// Algoritem Newtonove metode.
extern void celoten_prostor(double zacetni_cas);	// Algoritem iskanja po celotnem prostoru.
extern void powell(int *optimize_plot, double zacetni_cas);	// Algoritem Powellove metode.
extern void najstrmejsi_sestop(int *optimize_plot, double zacetni_cas);	// Algoritem metode najstrmejega sestopa.

extern double *predlagaj_novo_tocko();	// Funkcija pregleda parametrski prostor in glede na e izraunane toke predlaga nov zaeten poskus v toki, kjer parametrski prostor e ni bil preiskan.

extern struct wordlist **analize;	// Kazalec na polje analiz.
extern int stevilo_analiz;	// Velikost polja analiz.
extern struct wordlist **implicitne_omejitve;	// Kazalec na polje implicitnih omejitev.
extern int stevilo_implicitnih_omejitev;	// Velikost polja implicitnih omejitev.
extern struct parameter *parametri;	// Kazalec na polje parametrov.
extern int stevilo_parametrov;	// Velikost polja parametrov.
extern struct wordlist *kriterijska_funkcija;	// Kazalec na kriterijsko funkcijo.
extern struct metoda trenutna_metoda;	// Trenutno izbrana optimizacijska metoda.
extern double **rezultati;	// Kazalec na polje delnih rezultatov, ki se shranjujejo med tekom optimizacijskega postopka.
extern unsigned long int velikost_polja;	// Velikost trenutno rezerviranega polja rezultati.
extern unsigned long int najvecje_stevilo_iteracij;	// Optimizacijska zanka se prekine po doloenem tevilu iteracij, eprav optimizacijska metoda e ni konvergirala.
extern unsigned long int stevilo_iteracij;	// tevilo iteracij pomeni, kolikokrat so bile izvedene zahtevane analize.
extern double **izracunane_tocke;	// Polje e izraunanih tok v parametrskem prostoru.
extern unsigned long int stevilo_izracunanih_tock;	// Velikost polja e izraunanih tok.
extern int zacetni_poskus;	// Ute toke z najvejo vrednostjo kriterijske funkcije.
extern struct plot *plot_optimize;	// Kazalec na skupino vektorjev optimize, ki jo ustvari ukaz optimizacijski proces.
extern struct osebek *populacija;	// Zadnja populacija po konanem genetskem algoritmu. Naslednji klic genetskega algoritma nadaljuje iz te populacije, e obstaja.
extern double stop_cost; // Ustavitveni kriterij
extern int optimize_debug;	// Sporocila med izvajanjem zanke
extern double summary_send_time;
